<?php

namespace App\Http\Controllers;

use App\Models\Levels;
use App\Models\Topic;
use App\Models\Subject;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

class TopicController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('permission:view-topic')->only('index', 'getData');
        $this->middleware('permission:create-topic')->only('create', 'store');
        $this->middleware('permission:edit-topic')->only('edit', 'update');
        $this->middleware('permission:delete-topic')->only('destroy');
    }

    public function index()
    {
        $levels = Levels::all();
        $subjects = Subject::all();
        return view('topics.index', compact('levels', 'subjects'));
    }

    public function getData(Request $request)
    {
        // dd($request->all());
        $topics = Topic::orderBy('updated_at','desc');

        if ($request->level != '') {
            $topics->where('level_id', $request->level);
        }

        if ($request->subject != '') {
            $topics->where('subject_id', $request->subject);
        }

        $topics->get();

        return DataTables::of($topics)
        ->editColumn('level_id', function ($topics) {
            return Levels::find($topics->level_id) ? Levels::find($topics->level_id)->name :'' ;
        })
        ->editColumn('subject_id', function ($topics) {
            return Subject::find($topics->subject_id) ? Subject::find($topics->subject_id)->name :'' ;
        })
        ->editColumn('created_at', function ($admin) {
            return date('d-m-Y', strtotime($admin->created_at));
        })
        ->addColumn('updated_at', function ($admin) {
            return date('Y-m-d', strtotime($admin->updated_at));
        })
        ->addColumn('actions', function ($topics) {
            return view('topics.action',compact('topics'));
        })
        ->rawColumns(['actions'])->make(true);
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $subjects = Subject::all();
        $levels = Levels::all();
        return view('topics.create',compact('subjects','levels'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'level' => 'required',
            'subject' => 'required',
            'name' => 'required',
        ], [
            'name.required' => 'Name of Topic is required.'
        ]);
        $find = Topic::where('level_id', $request->level)->where('subject_id', $request->subject)->where('name',$request->name)->first();
        if($find != ''){
        return redirect()->back()->withErrors('Topic Already Exist On This Level & Subject!')->withInput();
        }
        $topic = new Topic ;
        $topic->level_id = $request->level;
        $topic->subject_id = $request->subject;
        $topic->name = $request->name;
        $topic->save();
        return redirect()->route('topic.index')->with('message', 'Topic created successfully');
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Topic  $topic
     * @return \Illuminate\Http\Response
     */
    public function show(Topic $topic)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Topic  $topic
     * @return \Illuminate\Http\Response
     */
    public function edit(Topic $topic)
    {
        $subjects = Subject::all();
        $levels = Levels::all();
        return view('topics.edit',compact('topic','subjects','levels'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Topic  $topic
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Topic $topic)
    {
        $this->validate($request, [
            'level' => 'required',
            'subject' => 'required',
            'name' => 'required',
        ], [
            'name.required' => 'Name of Topic is required.'
        ]);
        $find = Topic::where('level_id', $request->level)->where('subject_id', $request->subject)->where('name',$request->name)->where('id','!=',$topic->id)->first();
        if($find != ''){
        return redirect()->back()->withErrors('Topic Already Exist On This Level & Subject!')->withInput();
        }
        $topicUpdate = $topic ;
        $topicUpdate->level_id = $request->level;
        $topicUpdate->subject_id = $request->subject;
        $topicUpdate->name = $request->name;
        $topicUpdate->save();
        return redirect()->route('topic.index')->with('message', 'Topic Updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Topic  $topic
     * @return \Illuminate\Http\Response
     */
    public function destroy(Topic $topic)
    {
        $topic->delete() ;
        return redirect()->route('topic.index')->with('message', 'Topic Deleted successfully');
    }
}
